from osv import osv,fields
import time
from mx import DateTime
import datetime
import netsvc
from tools.translate import _

class purchase_freight(osv.osv):
    _name="purchase.freight"
    _description="Freight Forwarder"
    _columns={
        "code":fields.char("Code",size=32),
        "name":fields.char("Name",size=32,select=1),  
        "partner_id" : fields.many2one("res.partner","Partner"),
        "type" : fields.selection([('sea',"Sea"),('air','Air'),('truck','Truck')],"Type",select=1),
        "active" :fields.boolean("Active"),
    }
    _defaults={
        'active' : lambda *a : 1
    }
purchase_freight()

class res_partner(osv.osv):
    _inherit="res.partner"

    _columns={
        "property_partner_incoterm": fields.property(
            'stock.incoterms',
            type='many2one',
            relation='stock.incoterms',
            string="Incoterm",
            method=True,
            view_load=True
        ),
        "property_partner_freight": fields.property(
            'purchase.freight',
            type='many2one',
            relation='purchase.freight',
            string="Freight",
            method=True,
            view_load=True
        ),
    }
res_partner()

class purchase_requirement(osv.osv):

    """
    -Price comparison , find the best price
        1. Get sale order (inquiry stage)
        *Wizard : convert to Quotation Request(PR)
                    - select line and supplier
    User : "Saleman" send to Purchase > response to Sale Manager(result and quout) > quotation( adjust prices) to salesman or send direct to customer
    priority :
        1. Spec
        2. Price : cost of goods , delivery(freight and transport), payment term
        3. lead time
    """

    _name="purchase.requirement"

    def _get_date(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for date in self.browse(cr, uid, ids):
            if date.date_end and date.date_start:
                end = datetime.datetime.strptime(date.date_end, "%Y-%m-%d")
                start = datetime.datetime.strptime(date.date_start, "%Y-%m-%d")
                delta = end-start
                res[date.id]=delta.days
                if delta.seconds > 0 :
                    res[date.id]=delta.days + 1
            else:
                res[date.id] = None
        return res
    def _set_date(self, cr, uid, ids, name, value, arg, context):
        """Only here so to avoid storing the data in DB (would be redondant), 
        everything is fired in the target_discount_price_change event"""
        return True

    def date_change(self, cr, uid, ids, day=0,created_date=False,context={}):
        res = {}
        if day:
            date_planned = DateTime.strptime(created_date,'%Y-%m-%d') + DateTime.DateTimeDeltaFromDays(day or 0.0)
            date_planned = date_planned.strftime('%Y-%m-%d')
            res['value'] = {'date_end':date_planned}
        else:
            res['value'] = {'date_end':False}
        return res

    _columns={
        "name": fields.char("Ref no.", size=32,required=1,readonly=1, states={'draft':[('readonly',False)]},select=1),
        "date_start": fields.date("Date Start",readonly=1),
        "type" :fields.selection([('purchase',"Purchase"),('sale','Sale')],"Type",help="Specific type of purchase requirement: - Purchase Create for PO and Sale create for SO"),

        "date_end": fields.date("Date End",help="End of waiting" ,readonly=1, states={'draft':[('readonly',False)]}),
        "day_end":fields.function(_get_date,fnct_inv=_set_date,method=True,type="integer" ,string="Request Day",readonly=1,states={'draft':[('readonly',False)]}),
        "user_id": fields.many2one("res.users", "Salesman",readonly=1,select=1),
        #"so_id" :fields.many2one("sale.order" , "Sale Related" ,readonly=1),
        "customer_id" : fields.many2one("res.partner", "Customer", domain=[('customer','=',True)],readonly=1, states={'draft':[('readonly',False)]},required=1 ),
        "description": fields.text("Description"),
        "state": fields.selection([("draft","Draft"),('confirmed','Confirmed'),('done','Done'),('cancelled','Cancelled')], "State", readonly=1),
        "lines" : fields.one2many("purchase.requirement.line","prq_id","Lines",readonly=1, states={'draft':[('readonly',False)]}),
        "qr_ids" :fields.one2many("quotation.request","prq_id","quotation request",readonly=1,states={'wait':[('readonly',False)]}),
    }

    _defaults={
        "date_start": lambda *a: time.strftime("%Y-%m-%d"),
        "state": lambda *a: "draft",
        "name": lambda *a : "/",
        "user_id" : lambda obj,cr,uid,ctx : uid,
        "type" : lambda *a : 'sale',
    }

    #def create(self, cr, user, vals, context=None):
        #""" to create puchase reqiurement if old quotatoin still valid can get old price(almost in 3 day)
            #- support get old quotation request (state valid)     
        #"""
        #return super(purchase_requirement,self).create(cr, user, vals, context)

    def wkf_draft(self,cr,uid,ids,context={}):
        print "purchase_requirement: wkf_draft " , ids  
        for req in self.browse(cr,uid,ids):
            req.write({"state":"draft"})
        return True

    def wkf_confirm(self,cr,uid,ids,context={}):
        print "purchase_requirement: wkf_confirm " , ids  
        for req in self.browse(cr,uid,ids):
            name=req.name
            if ('PRQ' not in req.name ) or (req.name=='/'):
                name = self.pool.get('ir.sequence').get(cr, uid, 'purchase.requirement')
            req.write({"state":"confirmed",'name':name})
        return True

    def wkf_done(self,cr,uid,ids,context={}):
        """
            -it's can use only requirements fulfilled
        """
        print "purchase_requirement: wkf_done " , ids  
        for req in self.browse(cr,uid,ids):
            for line in req.lines:
                if not line.qr_line_ids :
                    raise osv.except_osv(_("Error"),_("You must select the supplier for every request line"))
            req.write({"state":"done"})
        return True

    def wkf_cancel(self,cr,uid,ids,context={}):
        self.write(cr,uid,ids , {"state":"cancelled"})
        return True

purchase_requirement()

class purchase_requirement_line(osv.osv):

    """
        -TODO :when select item in quotation request ,it will be updated price of purchase requirement
    """

    _name="purchase.requirement.line"

    def _subtotal(self,cr,uid,ids,name,args,context={}):
        vals={}
        for line in self.browse(cr,uid,ids):
            vals[line.id]=line.product_qty*line.price_unit
        return vals

    def onchange_product(self,cr,uid,ids,product_id):
        if not product_id:
            return {"value":{"name":None,"uom_id":None,"qty":None,"price_unit":None,"supplier_id":None}}
        prod=self.pool.get("product.product").browse(cr,uid,product_id)
        uom_id=prod.uom_id.id
        qty=0.0
        vals={
            "name": "[%s]:%s"%(prod.default_code,prod.name or ""),
            "variants" : prod.variants or "",
            "uom_id": uom_id,
        }
        #print "vals",vals
        return {"value": vals}

    _columns={
        "prq_id": fields.many2one("purchase.requirement","Requirement",ondelete="cascade"),
        "name": fields.char("Description",size=64,required=True,select=1 ),
        "variants": fields.char("Size",size=64,select=1 ),
        "product_id": fields.many2one("product.product","Product"),
        "product_qty": fields.float("Quantity",required=True),
        "uom_id": fields.many2one("product.uom","UoM",required=True),
        "notes" :fields.text("Note"),
        "qr_line_ids" :fields.one2many('quotation.request.line','prq_line_id','Quotation Request line' , readonly=1),
        'sol_id': fields.one2many('sale.order.line','prq_line_id','Sale Order line',readonly=1),
        'select_ok': fields.boolean("Select"),
    }

    _defaults={
    }

purchase_requirement_line()

class create_quotation_request_wizard(osv.osv_memory):
    """
        TODO :  use normal wizard : it's better than osv_memory
    """

    _name="create.quotation.request.wizard"
    _description="Wizard to Create  quotation request"

    #def _get_prq(self, cr, uid, ctx):
        #if ctx.get('action_id', False):
            #return ctx['action_id']
        #return False

    _columns={ 
        'prq_id':fields.many2one('purchase.requirement',"Purchase Requirement",readonly=1),
        'partner_ids' : fields.many2many('res.partner','res_partner_rel','td_id','partner_id',"Select Partner"),
    }

    _defaults={
        "prq_id" : lambda self,cr,uid,ctx : ctx.get('action_id',False),
    }

    def action_done(self,cr,uid,ids,ctx=None):
        qr_obj = self.pool.get("quotation.request")
        wf_service = netsvc.LocalService("workflow")
        prq_id = None
        for line in self.browse(cr,uid,ids):
            vals={}
            lines=[]
            prq_id = line.prq_id.id
            for prq in line.prq_id.lines:
                #print "prq" , prq
                lines.append((0,0,{
                    "name": prq.name or "", 
                    "variants": prq.product_id.variants or "",
                    "product_id" : prq.product_id.id,
                    "product_qty" : prq.product_qty,
                    "product_uom" : prq.uom_id.id,
                    "price_unit": 0.0,
                    "prq_line_id": prq.id,
                    })
                )
            #print line.partner_ids
            for partner in line.partner_ids:
                #print "get ", partner.id
                #print "get prq" ,line.prq_id.qr_ids
                for p in line.prq_id.qr_ids:
                    if partner.id == p.partner_id.id:
                        raise osv.except_osv("Error","Partner already created Quotation Request: %s"%partner.name)
                #vals['name']=
                vals['partner_id']=partner.id
                vals['partner_incoterm_id'] = partner.property_partner_incoterm.id
                vals['partner_freight_id'] = partner.property_partner_freight.id
                vals['partner_term'] =  partner.property_payment_term.id
                #vals['validity']=
                #vals["date_planned"]=
                vals["prq_id"] = prq_id
                vals["lines"]= lines
                vals["name"]= self.pool.get("ir.sequence").get(cr, uid, "quotation.request")
                vals["type"]= 'sale'

                #print vals
                qr_id = qr_obj.create(cr,uid,vals)
                #print qr_id

        return {'type': 'ir.actions.act_window_close'}
        #return {}

        #for osv_memory : when refresh page will run action again
        # and when exception occured it cannot reuse the old form 
        #return {
                #'view_type': 'form',
                #"view_mode": 'tree,form',
                #'res_model': 'quotation.request',
                #"domain": [("prq_id","=",prq_id)],
                #'type': 'ir.actions.act_window',
                #'target':'new',
        #}

create_quotation_request_wizard()

class quotation_request(osv.osv):
    """
        - support currency(supplier)
        - asst.sale will see only thai baht
        - Support list of Quotation Request of one purchase requirement
        - Allow disapprove pr line of Quotation Request/purchase requirement
        TODO : Support currency
    """
    _name="quotation.request"
    _description="quotation request"

    def _check_valid(self,cr,uid,ids,name,args,context={}):
        vals={}
        for req in self.browse(cr,uid,ids):
            #req.validity
            if not req.validity:
                vals[req.id]=False
                return vals
            valid_date = datetime.datetime.strptime(req.validity,'%Y-%m-%d')
            #datetime.datetime
            vals[req.id]=datetime.datetime<valid_date
        return vals

    def _total(self,cr,uid,ids,name,args,context={}):
        vals={}
        for req in self.browse(cr,uid,ids):
            amt=0.0
            for line in req.lines:
                amt+=line.price_subtotal
            vals[req.id]=amt
        return vals

    _columns={
        "name":fields.char("Doc No",size=32,readonly=1,states={'received':[('readonly',False)]},select=1),
        "ref" :fields.char("Quot No",size=32,readonly=1,states={'received':[('readonly',False)]}),
        "prq_id":fields.many2one("purchase.requirement","Purchase Requirement" , readonly=1),
        "partner_id":fields.many2one("res.partner","Supplier", readonly=1,states={'received':[('readonly',False)]} ,select=1),
        "partner_incoterm_id":fields.many2one("stock.incoterms","Incoterm", readonly=1,states={'received':[('readonly',False)]}),
        "partner_freight_id":fields.many2one("purchase.freight","Freight", readonly=1,states={'received':[('readonly',False)]}),
        "partner_term":fields.many2one("account.payment.term","Payment Term", readonly=1,states={'received':[('readonly',False)]}),
        "partner_delivery_term" : fields.char("Delivery Term",size=32, readonly=1,states={'received':[('readonly',False)]}),
        "lines": fields.one2many("quotation.request.line","qr_id","lines" , readonly=1,states={'received':[('readonly',False)]}),
        "date" :fields.date("Created Date",readonly=1,select=1),
        "date_planned" :fields.date("Scheduled Date" , readonly=1,states={'received':[('readonly',False)]}),
        "user_id" :fields.many2one("res.users","User Responsible",readonly=1,select=2),
        "validity" : fields.date("Price Valid" , readonly=1,states={'received':[('readonly',False)]}),
        "is_valid" : fields.function(_check_valid , method=True,string="Is Valid ?"),
        "total": fields.function(_total,method=True,store=True,type="float",string="Total"),
        "state":fields.selection([('draft','Draft'),('sent','Sent'),('received','Recieved'),('cancelled','Cancelled')],"State",readonly=1,select=1),
        "sequence":fields.integer('Rank'),
    }
    _order="sequence,total"

    _defaults={
        'date':lambda *a : time.strftime("%Y-%m-%d"), 
        "user_id" :lambda self,cr,uid,ctx: uid,
        "state": lambda *a : 'draft',
        "sequence" : lambda *a : 10,
    }

    def create(self, cr, user, vals, context=None):
        if ('name' not in vals) or (vals.get('name')=='/'):
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, 'quotation.request')
        return super(quotation_request,self).create(cr, user, vals, context)

    def button_send(self,cr,uid,ids,context={}):
        """
            request for quotation
        """
        print "button send :  Quotation Request"
        return self.write(cr,uid,ids,{'state':'sent'})

    def button_receive(self,cr,uid,ids,context={}):
        """
            - Comparing price
        """
        print "button open Quotation Request"
        return self.write(cr,uid,ids,{'state':'received'})

    def button_cancel(self,cr,uid,ids,context={}):
        print "button close Quotation Request"
        """
            finished select purchase requirement
        """
        return self.write(cr,uid,ids,{'state':'cancelled'})

quotation_request()

class quotation_request_line(osv.osv):

    """
        - some Quotation Request line , supplier may  has qty +/- ( production circumstances: lost , missing)
    """

    _name="quotation.request.line"
    _description="quotation request line"
    
    def _get_partner(self, cr, uid, ids, prop, unknow_none,unknow_dict):
        res = {}
        for line in self.browse(cr, uid, ids):
            res[line.id] = line.qr_id.partner_id.id
        return res

    def _amount_line(self, cr, uid, ids, prop, unknow_none,unknow_dict):
        res = {}
        #cur_obj=self.pool.get('res.currency')
        for line in self.browse(cr, uid, ids):
            #cur = line.order_id.pricelist_id.currency_id
            #res[line.id] = cur_obj.round(cr, uid, cur, line.price_unit * line.product_qty)
            res[line.id] = line.price_unit * line.product_qty
        return res


    _columns={
        "qr_id" : fields.many2one("quotation.request","Quotation Request" ,readonly=1 , ondelete='cascade'),
        "prq_line_id" : fields.many2one('purchase.requirement.line','PRQ Line', readonly=1,ondelete='cascade'),
        "name":fields.char("Description",size=64,readonly=0),
        "variants": fields.char("Size",size=64),
        "product_id" : fields.many2one("product.product", "Product", domain=[("purchase_ok","=",True)] ,readonly=0),
        "product_uom": fields.many2one("product.uom", "UoM",readonly=0),
        "price_unit" : fields.float("Unit Price", required=True, digits=(16,2)),
        "product_qty": fields.integer("Qty",size=7,readonly=0),
        "price_subtotal": fields.function(_amount_line, method=True, string="Subtotal",type="float",readonly=1),
        #"state": fields.selection([("selected","Selected"),("deselected","Deselected")],"State",readonly=1,select=1),
        "notes" : fields.text("Notes"),
        'select_ok' : fields.boolean('Select'),
        'related_partner' : fields.function(_get_partner,method=True,string="Supplier", type='many2one',relation="res.partner"),
    }

    #_order='price_unit'

    _defaults={
    }

    def change_select(self, cr, uid, ids, select_ok ,context={}):
        print "onchange select ", select_ok , ids,context
        for o in self.browse(cr,uid,ids):
            for line in o.prq_line_id:
                print 'change_select', line
        return {}

quotation_request_line()

class quotation_change_price_wizard(osv.osv_memory):
    """
        - Make it easy to change price in tree view
        button in list view :  gtk works but  web-5.0 doesn't work
    """
    _name="quotation.change.price.wizard"

    def _get_quotation(self, cr, uid, ctx):
        print "get quotation"
        # if quotation in close  , it should can't change
        qr_obj  = self.pool.get("quotation.request.line")
        #if qr_obj:
        #print ctx['action_id']

        for o in qr_obj.browse(cr,uid,[ctx['action_id']]):
            print 'o.state', o.qr_id.state
            if o.qr_id.state == 'close':
                raise osv.except_osv(_('Error'),_('You cannot change price of quotation request in state close'))

        if ctx.get('action_id', False):
            return ctx['action_id']
        return False

    _columns={
        'quotation_line_id' : fields.many2one('quotation.request.line',"Quotation Request Line",readonly=1),    
        'price' : fields.float("Price",digits=(8,2),required=1),    
    }

    _defaults={
        "user_id" : lambda self,cr,uid,ctx: uid,    
        "quotation_line_id" : _get_quotation,
    }

    def action_done(self,cr,uid,ids,ctx=None):
        tl_obj = self.pool.get("quotation.request.line")
        vals={}
        for line in self.browse(cr,uid,ids):
            vals["price_unit"] = line.price 
            print "Quotation request line update" , vals
            tl_obj.write(cr,uid,[line.quotation_line_id.id],vals)

        #return True 
        return {'type': 'ir.actions.act_window_close'}
        #return {
                #'view_type': 'form',
                #"view_mode": 'tree,form',
                #'res_model': 'requisition',
                #'type': 'ir.actions.act_window',
                #'target':'current',
        #}

quotation_change_price_wizard()

class sale_order(osv.osv):
    """
    """
    _inherit = "sale.order"

    def _requirements(self,cr,uid,ids,name,args,context={}):
        vals={}
        for order in self.browse(cr,uid,ids):
            order_ids=set([])
            for line in order.order_line:
                if line.prq_line_id:
                    order_ids.add(line.prq_line_id.prq_id.id)
            vals[order.id]=sorted(order_ids)
        return vals

    _columns = {
        'state': fields.selection([
            ('draft', 'Inquiry'),
            ('quote','Draft Quotation'),
            ('wait_mgr','Waiting mgr for Approve'),
            ('approved_quot', 'Approved Quotation'),
            ('waiting_date', 'Waiting Schedule'),
            ('manual', 'Manual In Progress'),
            ('progress', 'In Progress'),
            ('shipping_except', 'Shipping Exception'),
            ('invoice_except', 'Invoice Exception'),
            ('done', 'Done'),
            ('cancel', 'Cancelled')
            ], 'Order State', readonly=True, help="Gives the state of the quotation or sale order. The exception state is automatically set when a cancel operation occurs in the invoice validation (Invoice Exception) or in the packing list process (Shipping Exception). The 'Waiting Schedule' state is set when the invoice is confirmed but waiting for the scheduler to run on the date 'Date Ordered'.", select=True),
        "planned_date":fields.date("Delivery Date"),
        "prq_ids" :fields.function(_requirements, method=True, type="many2many", relation="purchase.requirement",string="Related Requirements" , readonly=1),
        "validity": fields.integer("Validity (days)"),
        'order_line': fields.one2many('sale.order.line', 'order_id', 'Order Lines', readonly=True, states={'draft': [('readonly', False)],'quote': [('readonly', False)]}),

        'sent':fields.boolean('Sent to customer',readonly=0),
        'rejected':fields.boolean('Rejected by customer',readonly=0),
    }

    _defaults = {
        'name': lambda obj, cr, uid, context : '/',
    }

    def wkf_quote(self, cr, uid, ids, context=None):
        print "wkf_quote"
        self.write(cr, uid, ids, {'state': 'quote'})
        return True

    def action_requirement_create(self, cr, uid, ids, *args):
        print "action_requirement_create"
        """ create purchase requirement for selected order line"""
        self.write(cr, uid, ids, {'state': 'requirement'})
        sol_obj = self.pool.get("sale.order.line")
        prq_ids = None 
        for o in self.browse(cr,uid,ids):
            line_ids = [l.id for l in o.order_line if l.req_ok]
            prq_ids = sol_obj.make_prq(cr,uid,line_ids)
        return prq_ids[0]

    def button_send(self, cr, uid, ids, *args):
        self.write(cr,uid,ids,{'sent':True})
        #wfs=netsvc.LocalService("workflow")
        #wfs.trg_validate(uid,"sale.order",ids,'button_disapprove',cr)
        return True

    def button_reject(self, cr, uid, ids, *args):
        """
            - customer bargain , or want to cancel
        """
        self.write(cr,uid,ids,{'rejected':True})
        if type(ids)==type([]):
            ids=ids[0]
        wfs=netsvc.LocalService("workflow")
        wfs.trg_validate(uid,"sale.order",ids,'button_revise',cr)
        return True

    def button_disapprove(self, cr, uid, ids, *args):
        print "button_disapprove",ids
        #self.write(cr,uid,ids,{'disapproved':True})
        wfs=netsvc.LocalService("workflow")
        if type(ids)==type([]):
            ids=ids[0]
        wfs.trg_validate(uid,"sale.order",ids,'button_disapprove',cr)
        return True

    def button_revise(self, cr, uid, ids, *args):
        """
        - posible for update price
        """
        print "button_revise",ids
        #self.write(cr,uid,ids,{'revised':True})
        #self.write(cr,uid,ids,{'disapproved':True})
        wfs=netsvc.LocalService("workflow")
        if type(ids)==type([]):
            ids=ids[0]
        wfs.trg_validate(uid,"sale.order",ids,'button_revise',cr)
        return True

    def wkf_wait_mgr(self, cr, uid, ids, context=None):
        """
        - after requirement finish sale manager need to mark up sale price
        """
        print 'wkf_wait_mgr' ,ids
        # create new sequence(quotation)
        for order in self.browse(cr,uid,ids):
            #print 'order.name' , order.name
            name = order.name
            if 'IN' in order.name or order.name == '/':
                name = self.pool.get('ir.sequence').get(cr, uid, 'sale.order')
            order.write({'state': 'wait_mgr','name':name,'origin':order.name})

            if not order.order_line:
                raise osv.except_osv(_('Error !'),_('No line item in inquiry'))
        return True

    def wkf_approved_quot(self, cr, uid, ids, context=None):
        """
        - after sale manager mark-up sale price finished
        """
        vals ={
            'state':'approved_quot',
        }
        self.write(cr, uid, ids, vals)
        return True

    def unlink(self, cr, uid, ids, context=None):
        sale_orders = self.read(cr, uid, ids, ['state'])
        unlink_ids = []
        for s in sale_orders:
            if s['state'] not in ['manual','progress']:
                unlink_ids.append(s['id'])
            else:
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete Sale Order(s) which are already confirmed !'))
        return osv.osv.unlink(self, cr, uid, unlink_ids, context=context)

    def create(self, cr, uid, vals, context=None):
        if ('name' not in vals) or (vals.get('name')=='/'):
            #vals['name'] = self.pool.get('ir.sequence').get(cr, uid, 'sale.order')
            vals['name'] = self.pool.get('ir.sequence').get(cr, uid, 'sale.inquiry')
        return super(sale_order,self).create(cr, uid, vals, context)

    def copy(self, cr, uid, id, default=None, context={}):
        if not default:
            default = {}
        default.update({
            'state': 'draft',
            'shipped': False,
            'invoice_ids': [],
            'picking_ids': [],
            'name': '/',
            "pr_ids" : [],
            "prq_ids" :[],
        })
        return osv.osv.copy(self,cr, uid, id, default, context)

    def action_wait(self, cr, uid, ids, *args):
        for o in self.browse(cr,uid,ids):
            vals = {
                'origin': (o.origin and o.origin+' , ') + o.name ,
                'name':self.pool.get('ir.sequence').get(cr, uid, 'sale.order.confirm'),
                }
            o.write(vals)
        return super(sale_order,self).action_wait(cr,uid,ids,args)

sale_order()

class sale_order_line(osv.osv):
    _inherit="sale.order.line"
    _columns={
        'prq_line_id': fields.many2one('purchase.requirement.line',"Purchase Requirement Line",readonly=1),
    }

    def copy_data(self, cr, uid, id, default=None, context={}):
        if not default:
            default = {}
        default.update({'state': 'draft', 'prq_line_id':[]})
        return super(sale_order_line, self).copy_data(cr, uid, id, default, context)

    def make_prq(self,cr,uid,ids,day=0,context=None):
        """
            -create purchase requirement from sale order line
        """

        dp = DateTime.today() + DateTime.DateTimeDeltaFromDays(day or 0.0)

        date_planned = dp.strftime('%Y-%m-%d')

        group_lines={}
        prq_ids=[]
        # Group sale order line by 
        for line in self.browse(cr,uid,ids):
            #line.write({'state':'req_price'})
            k=(line.order_id.partner_id.id)
            group_lines.setdefault(k,[]).append(line)

        for (cust_id),lines in group_lines.items():
            prq_lines=[]
            origin=set([])

            for line in lines:
                #TODO : sale order line must be selected
                if line.state!="draft": #should use state inquiry
                    continue
                prq_lines.append((0,0,{
                    "name": line.name or "", 
                    "variants": line.product_id.variants or "",
                    "product_id" : line.product_id.id or "",
                    "product_qty" : line.product_uom_qty,
                    "uom_id" : line.product_uom.id,
                    "sol_id" : [(6,0,[line.id])]
                    #"notes" : line.notes,
                }))

                origin.add(line.order_id.name)

            vals={
                "description": ",".join(sorted(origin)),
                "lines": prq_lines,
                "qr_ids":[], #TODO: can also create Quotation Request 
                "customer_id": line.order_id.partner_id.id,
                "date_end": date_planned,
            }

            print 'purch_req' , vals
            prq_id=self.pool.get("purchase.requirement").create(cr,uid,vals)
            print "prq_id ", prq_id
            #line.write({'prq_line_id':prq_id})
            prq_ids.append(prq_id)

        #print 'prq_ids' , prq_ids
        return prq_ids #just return ids for wizard 

    def button_select(self,cr,uid,ids,context={}):
        """
            # select sol to create purchase requirement
        """
        if self.browse(cr,uid,ids)[0].req_ok:
            return self.write(cr,uid,ids,{'req_ok':False})
        return self.write(cr,uid,ids,{'req_ok':True})

sale_order_line()

class purchase_order(osv.osv):
    _inherit="purchase.order"

    _columns={
        'state': fields.selection([ ('draft', 'Request for Quotation'), 
                                    ('wait', 'Waiting'), 
                                    ('confirmed', 'Confirmed'),
                                    ('approved', 'Approved'),
                                    ('except_picking', 'Shipping Exception'), 
                                    ('except_invoice', 'Invoice Exception'), 
                                    ('done', 'Done'), 
                                    ('cancel', 'Cancelled')],'Order Status', readonly=True, help="The state of the purchase order or the quotation request. A quotation is a purchase order in a 'Draft' state. Then the order has to be confirmed by the user, the state switch to 'Confirmed'. Then the supplier must confirm the order to change the state to 'Approved'. When the purchase order is paid and received, the state becomes 'Done'. If a cancel action occurs in the invoice or in the reception of goods, the state becomes in exception.", select=True),
    }
#purchase_order()

